#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora.panchanga import drik
chart_1_dob = (2000,4,9)
chart_1_tob = (17,55,0)
chart_1_place = drik.Place('unknown',42+30/60,-71-12/60,-4.0)
chart_1_dcf = 1

chart_2_dob = chart_1_dob
chart_2_tob = chart_1_tob
chart_2_place = chart_1_place
chart_2_dcf = 16
# A.B Vajpayee
chart_3_dob = (1926,12,25)
chart_3_tob = (5,12,0)
chart_3_place = drik.Place('unknown',26+14/60,78+10/60,5.5)
chart_3_dcf = 1

chart_4_dob = (1968,3,19)
chart_4_tob = (18,30,0)
chart_4_place = drik.Place('unknown',17+7/60,82+19/60,5.5)
chart_4_dcf = 1

chart_5_dob = (2000,4,19)
chart_5_tob = (19,21,0)
chart_5_place = drik.Place('unknown',42+40/60,-71-8/60,-4.0)
chart_5_dcf = 9
# PV Narasimha Rao
chart_6_dob = (1921,6,21)
chart_6_tob = (12,49,0)
chart_6_place = drik.Place('unknown',18+26/60,79+9/60,5+17/60)
chart_6_dcf = 1
# Ronald Regan
chart_7_dob = (1911,2,6)
chart_7_tob = (2,4,0)
chart_7_place = drik.Place('unknown',41+38/60,-89-47/60,-6.0)
chart_7_dcf = 1

chart_8_dob = (1946,12,2)
chart_8_tob = (6,45,0)
chart_8_place = drik.Place('unknown',38+6/60,15+39/60,1.0)
chart_8_dcf = 1
#Chatrapati Sivaji
chart_9_dob = (1630,2,19)
chart_9_tob = (18,26,0)
chart_9_place = drik.Place('unknown',18+32/60,73+53/60,5.5)
chart_9_dcf = 1 # 9
# Akbar
chart_10_dob = (1542,12,4)
chart_10_tob = (3,39,0)
chart_10_place = drik.Place('unknown',25+19/60,69+47/60,5.0)
chart_10_dcf = 1

chart_11_dob = (1921,6,28)
chart_11_tob = (13,8,0)
chart_11_place = drik.Place('unknown',18+26/60,79+9/60,5.0)
chart_11_dcf = 1

chart_12_dob = (1958,8,16)
chart_12_tob = (7,5,0)
chart_12_place = drik.Place('unknown',43+36/60,-83-53/60,-4.0)
chart_12_dcf = 10

chart_14_dob = (1944,8,20)
chart_14_tob = (7,11,0)
chart_14_place = drik.Place('unknown',18+58/60,72+49/60,5.5)
chart_14_dcf = 3

#Bill Cosby
example_27_dob = (1937,7,12)
example_27_tob = (0,30,0)
example_27_place = drik.Place('unknown',39+57/60,-75-10/60,-5.0)
example_27_dcf = 3
